#include <math.h>
#include "main.h"
#include "game.h"
#include "collide.h"



static void do_corner_collision(POD *pod, float xc, float yc)
{
	float xd = pod->x - xc;
	float yd = pod->y - yc;
	float d = xd*xd + yd*yd;
	if (d < (float)FIXED_POD_RADIUS*FIXED_POD_RADIUS) {
		d = 1.0f / (float)sqrt(d);
		xd *= d;
		yd *= d;
		pod->x = (pod->x & ~0xFFFF) | ((int)(xd * FIXED_POD_RADIUS) & 0xFFFF);
		pod->y = (pod->y & ~0xFFFF) | ((int)(yd * FIXED_POD_RADIUS) & 0xFFFF);
		d = xd*pod->xv + yd*pod->yv;
		if (d < 0) {
			xd *= 2*d;
			yd *= 2*d;
			pod->xv = (int)(pod->xv - xd);
			pod->yv = (int)(pod->yv - yd);
		}
	}
}



void detect_wall_collisions(GAME *game, POD *pod)
{
	int xi = pod->x >> 16, yi = pod->y >> 16;
	int xf = pod->x & 0xFFFF, yf = pod->y & 0xFFFF;

	int collided = 0;

	if (xf < FIXED_POD_RADIUS) {
		if (xi <= 0 || TILE_IS_HIGH(game->map[yi][xi-1])) {
			pod->x = (pod->x & ~0xFFFF) | FIXED_POD_RADIUS;
			if (pod->xv < 0) pod->xv = -pod->xv;
			collided = 1;
		}
	} else if (xf > 0x10000 - FIXED_POD_RADIUS) {
		if (xi >= game->map_w - 1 || TILE_IS_HIGH(game->map[yi][xi+1])) {
			pod->x = (pod->x & ~0xFFFF) | (0x10000 - FIXED_POD_RADIUS);
			if (pod->xv > 0) pod->xv = -pod->xv;
			collided = 1;
		}
	}

	if (yf < FIXED_POD_RADIUS) {
		if (yi <= 0 || TILE_IS_HIGH(game->map[yi-1][xi])) {
			pod->y = (pod->y & ~0xFFFF) | FIXED_POD_RADIUS;
			if (pod->yv < 0) pod->yv = -pod->yv;
			collided = 1;
		}
	} else if (yf > 0x10000 - FIXED_POD_RADIUS) {
		if (yi >= game->map_h - 1 || TILE_IS_HIGH(game->map[yi+1][xi])) {
			pod->y = (pod->y & ~0xFFFF) | (0x10000 - FIXED_POD_RADIUS);
			if (pod->yv > 0) pod->yv = -pod->yv;
			collided = 1;
		}
	}

	if (collided) return;

	if (xf < FIXED_POD_RADIUS) {
		if (yf < FIXED_POD_RADIUS) {
			if (TILE_IS_HIGH(game->map[yi-1][xi-1])) do_corner_collision(pod, xi << 16, yi << 16);
		} else if (yf > 0x10000 - FIXED_POD_RADIUS) {
			if (TILE_IS_HIGH(game->map[yi+1][xi-1])) do_corner_collision(pod, xi << 16, (yi << 16) + 0x10000);
		}
	} else if (xf > 0x10000 - FIXED_POD_RADIUS) {
		if (yf < FIXED_POD_RADIUS) {
			if (TILE_IS_HIGH(game->map[yi-1][xi+1])) do_corner_collision(pod, (xi << 16) + 0x10000, yi << 16);
		} else if (yf > 0x10000 - FIXED_POD_RADIUS) {
			if (TILE_IS_HIGH(game->map[yi+1][xi+1])) do_corner_collision(pod, (xi << 16) + 0x10000, (yi << 16) + 0x10000);
		}
	}
}



void detect_pod_collision(GAME *game, POD *pod1, POD *pod2)
{
	float xd = pod2->x - pod1->x;
	float yd = pod2->y - pod1->y;
	float d = xd*xd + yd*yd;
	if (d < (float)FIXED_POD_RADIUS*FIXED_POD_RADIUS*4.0f) {
		d = sqrt(d);
		xd /= d;
		yd /= d;
		d = FIXED_POD_RADIUS - d*0.5f;
		pod1->x -= (int)(xd * d);
		pod1->y -= (int)(yd * d);
		pod2->x += (int)(xd * d);
		pod2->y += (int)(yd * d);
		d = xd*(pod2->xv-pod1->xv) + yd*(pod2->yv-pod1->yv);
		if (d < 0) {
			xd *= d;
			yd *= d;
			pod1->xv = (int)(pod1->xv + xd);
			pod1->yv = (int)(pod1->yv + yd);
			pod2->xv = (int)(pod2->xv - xd);
			pod2->yv = (int)(pod2->yv - yd);
		}
		detect_wall_collisions(game, pod1);
		detect_wall_collisions(game, pod2);
	}
}
