#include <math.h>
#include "main.h"
#include "game.h"
#include "bsp.h"
#include "drawpod.h"



#define POD_RADII 8
#define POD_LEVELS 7


#define R POD_RADIUS
#define H POD_HEIGHT
static float rs[POD_LEVELS] = {R*0.2f, R*0.3f, R*0.2f, R*0.8f, R*0.933333333f, R     , R*0.933333333f};
static float hs[POD_LEVELS] = {H*1.2f, H*1.0f, H*0.8f, H*0.2f, H*0.2f        , H*0.1f,   0           };
#undef H
#undef R

#define HEIGHT_0 (POD_HEIGHT * 1.3f)
#define OUTERMOST_METAL (POD_RADIUS * 0.8f)

static int radii_order[POD_RADII] = {0, 4, 2, 6, 1, 5, 3, 7};


void init_pod_polygons(POD *pod)
{
	int i, j;
	for (i = 0; i < POD_RADII; i++) {
		pod->polygon[i].n_vtx = 3;
		pod->polygon[i].p = &pod->polyvtx[3*i];
		pod->polyvtx[3*i  ].vtx = &pod->vertex[0];
		pod->polyvtx[3*i+1].vtx = &pod->vertex[1+i];
		pod->polyvtx[3*i+2].vtx = &pod->vertex[1+((i+1)&(POD_RADII-1))];
	}
	for (; i < POD_RADII*POD_LEVELS; i++) {
		pod->polygon[i].n_vtx = 4;
		pod->polygon[i].p = &pod->polyvtx[4*i-POD_RADII];
		pod->polyvtx[4*i-POD_RADII  ].vtx = &pod->vertex[1-POD_RADII+i];
		pod->polyvtx[4*i-POD_RADII+1].vtx = &pod->vertex[1+i];
		pod->polyvtx[4*i-POD_RADII+2].vtx = &pod->vertex[1+(((i+1)&(POD_RADII-1))|(i&~(POD_RADII-1)))];
		pod->polyvtx[4*i-POD_RADII+3].vtx = &pod->vertex[1-POD_RADII+(((i+1)&(POD_RADII-1))|(i&~(POD_RADII-1)))];
	}
	pod->polygon[POD_RADII*POD_LEVELS].n_vtx = 8;
	pod->polygon[POD_RADII*POD_LEVELS].p = &pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII  ].vtx = &pod->vertex[1+POD_RADII*(POD_LEVELS-1)  ];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+1].vtx = &pod->vertex[1+POD_RADII*(POD_LEVELS-1)+7];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+2].vtx = &pod->vertex[1+POD_RADII*(POD_LEVELS-1)+6];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+3].vtx = &pod->vertex[1+POD_RADII*(POD_LEVELS-1)+5];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+4].vtx = &pod->vertex[1+POD_RADII*(POD_LEVELS-1)+4];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+5].vtx = &pod->vertex[1+POD_RADII*(POD_LEVELS-1)+3];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+6].vtx = &pod->vertex[1+POD_RADII*(POD_LEVELS-1)+2];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+7].vtx = &pod->vertex[1+POD_RADII*(POD_LEVELS-1)+1];

	for (i = 0; i < 6; i++) {
		pod->polygon[POD_RADII*POD_LEVELS+1+i].n_vtx = 4;
		pod->polygon[POD_RADII*POD_LEVELS+1+i].p = &pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+8+i*4];
		pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+8+i*4  ].vtx = &pod->vertex[1+POD_RADII*POD_LEVELS+i];
		pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+8+i*4+1].vtx = &pod->vertex[1+POD_RADII*POD_LEVELS+6+i];
		pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+8+i*4+2].vtx = &pod->vertex[1+POD_RADII*POD_LEVELS+6+(i+1)%6];
		pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+8+i*4+3].vtx = &pod->vertex[1+POD_RADII*POD_LEVELS+(i+1)%6];
	}
	pod->polygon[POD_RADII*POD_LEVELS+1+6].n_vtx = 6;
	pod->polygon[POD_RADII*POD_LEVELS+1+6].p = &pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+8+6*4];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+8+6*4  ].vtx = &pod->vertex[1+POD_RADII*POD_LEVELS  ];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+8+6*4+1].vtx = &pod->vertex[1+POD_RADII*POD_LEVELS+1];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+8+6*4+2].vtx = &pod->vertex[1+POD_RADII*POD_LEVELS+2];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+8+6*4+3].vtx = &pod->vertex[1+POD_RADII*POD_LEVELS+3];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+8+6*4+4].vtx = &pod->vertex[1+POD_RADII*POD_LEVELS+4];
	pod->polyvtx[4*POD_RADII*POD_LEVELS-POD_RADII+8+6*4+5].vtx = &pod->vertex[1+POD_RADII*POD_LEVELS+5];

	pod->polygon[POD_RADII*POD_LEVELS].polytype = POLYTYPE_PTEX;
	pod->polygon[POD_RADII*POD_LEVELS].texture = dat[GFX_TEX_POD_RUBBER_0 + pod->id].dat;
	{
		float os = (float)sin(0) * 0.4999f;
		float oc = (float)cos(0) * 0.4999f;
		for (i = 0; i < POD_RADII; i++) {
			float ns = (float)sin((i-15) * (3.14159265358979323846f / (POD_RADII>>1))) * 0.4999f;
			float nc = (float)cos((i-15) * (3.14159265358979323846f / (POD_RADII>>1))) * 0.4999f;
			{
				POLYGON *polygon = &pod->polygon[i];
				polygon->texture = dat[GFX_TEX_POD_RUBBER_0 + pod->id].dat;
				polygon->polytype = POLYTYPE_PTEX;
				polygon->p[0].u = 0.5f; polygon->p[0].v = 0.5f;
				polygon->p[1].u = 0.5f - os*rs[0]; polygon->p[1].v = 0.5f - oc*rs[0];
				polygon->p[2].u = 0.5f - ns*rs[0]; polygon->p[2].v = 0.5f - nc*rs[0];
			}
			for (j = 1; j < POD_LEVELS; j++) {
				POLYGON *polygon = &pod->polygon[j*POD_RADII+i];
				polygon->texture = dat[GFX_TEX_POD_RUBBER_0 + pod->id].dat;
				polygon->polytype = POLYTYPE_PTEX;
				if (j <= 3) {
					polygon->p[0].u = 0.5f - os/OUTERMOST_METAL*rs[j-1]; polygon->p[0].v = 0.5f - oc/OUTERMOST_METAL*rs[j-1];
					polygon->p[1].u = 0.5f - os/OUTERMOST_METAL*rs[j  ]; polygon->p[1].v = 0.5f - oc/OUTERMOST_METAL*rs[j  ];
					polygon->p[2].u = 0.5f - ns/OUTERMOST_METAL*rs[j  ]; polygon->p[2].v = 0.5f - nc/OUTERMOST_METAL*rs[j  ];
					polygon->p[3].u = 0.5f - ns/OUTERMOST_METAL*rs[j-1]; polygon->p[3].v = 0.5f - nc/OUTERMOST_METAL*rs[j-1];
				} else {
					polygon->p[0].u = 0.5f - os/POD_RADIUS*rs[j-1]; polygon->p[0].v = 0.5f - oc/POD_RADIUS*rs[j-1];
					polygon->p[1].u = 0.5f - os/POD_RADIUS*rs[j  ]; polygon->p[1].v = 0.5f - oc/POD_RADIUS*rs[j  ];
					polygon->p[2].u = 0.5f - ns/POD_RADIUS*rs[j  ]; polygon->p[2].v = 0.5f - nc/POD_RADIUS*rs[j  ];
					polygon->p[3].u = 0.5f - ns/POD_RADIUS*rs[j-1]; polygon->p[3].v = 0.5f - nc/POD_RADIUS*rs[j-1];
				}
			}
			pod->polygon[POD_RADII*POD_LEVELS].p[(POD_RADII-i)&(POD_RADII-1)].u = 0.5f - os/POD_RADIUS*rs[POD_LEVELS-1];
			pod->polygon[POD_RADII*POD_LEVELS].p[(POD_RADII-i)&(POD_RADII-1)].v = 0.5f - oc/POD_RADIUS*rs[POD_LEVELS-1];
			os = ns; oc = nc;
		}
	}

	for (i = 0; i < 6; i++) {
		POLYGON *polygon = &pod->polygon[POD_RADII*POD_LEVELS+1+i];
		polygon->texture = dat[GFX_TEX_POD_CANNON_SIDE].dat;
		polygon->polytype = POLYTYPE_PTEX;
		polygon->p[0].u = 0.9999f; polygon->p[0].v = i * (1.0f / 6.0f);
		polygon->p[1].u = 0.0001f; polygon->p[1].v = i * (1.0f / 6.0f);
		polygon->p[2].u = 0.0001f; polygon->p[2].v = (i + 1) * (1.0f / 6.0f);
		polygon->p[3].u = 0.9999f; polygon->p[3].v = (i + 1) * (1.0f / 6.0f);
	}
	{
		POLYGON *polygon = &pod->polygon[POD_RADII*POD_LEVELS+1+6];
		polygon->texture = dat[GFX_TEX_POD_CANNON_END].dat;
		polygon->polytype = POLYTYPE_PTEX;
		polygon->p[0].u = 0.2501f; polygon->p[0].v = 0.0001f;
		polygon->p[1].u = 0.0001f; polygon->p[1].v = 0.5000f;
		polygon->p[2].u = 0.2501f; polygon->p[2].v = 0.9999f;
		polygon->p[3].u = 0.7499f; polygon->p[3].v = 0.9999f;
		polygon->p[4].u = 0.9999f; polygon->p[4].v = 0.5000f;
		polygon->p[5].u = 0.7499f; polygon->p[5].v = 0.0001f;
	}
}



void add_pod_polygons(BSP_TREE *tree, POD *pod)
{
	if (POD_IS_THERE(pod) && !pod->cloak_time) {
		int i, j;
		float xo = fixtof(pod->x), yo = fixtof(pod->y), zo = fixtof(pod->z);
		float phi = (float)pod->phi * (3.14159265358979323846f / (128 << 16));
		float sp = sin(phi), cp = cos(phi);
		float xf = sp, yf = -cp;
		float xr = cp, yr = sp;

		pod->vertex[0].x = xo;
		pod->vertex[0].y = yo;
		pod->vertex[0].z = zo + HEIGHT_0;
		for (i = 0; i < POD_RADII; i++) {
			float s = sin(i * (3.14159265358979323846f / (POD_RADII>>1)));
			float c = cos(i * (3.14159265358979323846f / (POD_RADII>>1)));
			float xd = xf*c - xr*s;
			float yd = yf*c - yr*s;
			for (j = 0; j < POD_LEVELS; j++) {
				pod->vertex[j*POD_RADII+i+1].x = xo + xd * rs[j];
				pod->vertex[j*POD_RADII+i+1].y = yo + yd * rs[j];
				pod->vertex[j*POD_RADII+i+1].z = zo + hs[j];
			}
		}

		pod->vertex[1+POD_RADII*POD_LEVELS  ].x = xo + xf * POD_RADIUS + xr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS  ].y = yo + yf * POD_RADIUS + yr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS  ].z = zo + 1.1f*POD_HEIGHT;
		pod->vertex[1+POD_RADII*POD_LEVELS+1].x = xo + xf * POD_RADIUS + xr * (0.10f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+1].y = yo + yf * POD_RADIUS + yr * (0.10f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+1].z = zo + 1.0f*POD_HEIGHT;
		pod->vertex[1+POD_RADII*POD_LEVELS+2].x = xo + xf * POD_RADIUS + xr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+2].y = yo + yf * POD_RADIUS + yr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+2].z = zo + 0.9f*POD_HEIGHT;
		pod->vertex[1+POD_RADII*POD_LEVELS+3].x = xo + xf * POD_RADIUS - xr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+3].y = yo + yf * POD_RADIUS - yr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+3].z = zo + 0.9f*POD_HEIGHT;
		pod->vertex[1+POD_RADII*POD_LEVELS+4].x = xo + xf * POD_RADIUS - xr * (0.10f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+4].y = yo + yf * POD_RADIUS - yr * (0.10f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+4].z = zo + 1.0f*POD_HEIGHT;
		pod->vertex[1+POD_RADII*POD_LEVELS+5].x = xo + xf * POD_RADIUS - xr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+5].y = yo + yf * POD_RADIUS - yr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+5].z = zo + 1.1f*POD_HEIGHT;

		pod->vertex[1+POD_RADII*POD_LEVELS+ 6].x = xo + xr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+ 6].y = yo + yr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+ 6].z = zo + 1.1f*POD_HEIGHT;
		pod->vertex[1+POD_RADII*POD_LEVELS+ 7].x = xo + xr * (0.10f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+ 7].y = yo + yr * (0.10f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+ 7].z = zo + 1.0f*POD_HEIGHT;
		pod->vertex[1+POD_RADII*POD_LEVELS+ 8].x = xo + xr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+ 8].y = yo + yr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+ 8].z = zo + 0.9f*POD_HEIGHT;
		pod->vertex[1+POD_RADII*POD_LEVELS+ 9].x = xo - xr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+ 9].y = yo - yr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+ 9].z = zo + 0.9f*POD_HEIGHT;
		pod->vertex[1+POD_RADII*POD_LEVELS+10].x = xo - xr * (0.10f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+10].y = yo - yr * (0.10f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+10].z = zo + 1.0f*POD_HEIGHT;
		pod->vertex[1+POD_RADII*POD_LEVELS+11].x = xo - xr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+11].y = yo - yr * (0.05f*POD_RADIUS);
		pod->vertex[1+POD_RADII*POD_LEVELS+11].z = zo + 1.1f*POD_HEIGHT;

		for (i = 0; i < POD_RADII; i++)
			pod->polygon[3*POD_RADII+i].texture = pod->invinc_time ? the_fractal_bitmap : dat[GFX_TEX_POD_METAL].dat;

		for (i = 0; i < POD_RADII; i++) {
			for (j = 0; j < POD_LEVELS; j++) {
				POLYGON *polygon = &pod->polygon[j*POD_RADII+radii_order[i]];
				prepare_polygon(polygon);
				add_temp_to_bsp_tree(tree, polygon);
			}
		}
		for (i = 0; i < 1+6+1; i++) {
			POLYGON *polygon = &pod->polygon[POD_RADII*POD_LEVELS+i];
			prepare_polygon(polygon);
			add_temp_to_bsp_tree(tree, polygon);
		}
	}
}



void prepare_pod_vertices(MATRIX_f *camera, POD *pod)
{
	if (POD_IS_THERE(pod) && !pod->cloak_time) {
		int i;
		for (i = 0; i < 1+POD_RADII*POD_LEVELS+2*6; i++) draw_prepare_vertex(camera, &pod->vertex[i]);
	}
}
