#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "drawpwrp.h"


/*           0
 *
 *           3         <-- back
 *    4             2
 *      5         1     <-- front
 *
 *      9         8    <-- back
 *   10             7
 *           6          <-- front
 *
 *          11
 */


static const unsigned char pwrpvtx[20][3] = {
	{0,1,2}, {0,2,3}, {0,3,4}, {0,4,5}, {0,5,1},
	{5,6,1}, {1,7,2}, {2,8,3}, {3,9,4}, {4,10,5},
	{1,6,7}, {2,7,8}, {3,8,9}, {4,9,10}, {5,10,6},
	{6,11,7}, {7,11,8}, {8,11,9}, {9,11,10}, {10,11,6}
};


void init_powerup_polygons(POWERUP *powerup)
{
	int i;
	for (i = 0; i < 20; i++) {
		powerup->polygon[i].n_vtx = 3;
		powerup->polygon[i].polytype = POLYTYPE_PTEX_TRANS;
		powerup->polygon[i].texture = dat[GFX_TEX_POWERUP].dat;
		powerup->polygon[i].p = &powerup->polyvtx[i*3];
		powerup->polyvtx[i*3  ].vtx = &powerup->vertex[pwrpvtx[i][0]];
		powerup->polyvtx[i*3+1].vtx = &powerup->vertex[pwrpvtx[i][1]];
		powerup->polyvtx[i*3+2].vtx = &powerup->vertex[pwrpvtx[i][2]];
	}
}


void add_powerup_polygons(BSP_TREE *tree, POWERUP *powerup)
#define RACK_Z 0.4f
#define RACK_R 0.9f
{
	float phi = powerup->frame * (3.14159265358979323846f*2/POWERUP_N_FRAMES);
	int i;

	powerup->vertex[0].x = powerup->x + 0.5f;
	powerup->vertex[0].y = powerup->y + 0.5f;
	powerup->vertex[0].z = POWERUP_RADIUS * 2;

	for (i = 0; i < 5; i++) {
		powerup->vertex[1+i].x = powerup->x + 0.5f + POWERUP_RADIUS * RACK_R * sin(phi + 3.14159265358979323846f/5 * (1+2*i));
		powerup->vertex[1+i].y = powerup->y + 0.5f + POWERUP_RADIUS * RACK_R * cos(phi + 3.14159265358979323846f/5 * (1+2*i));
		powerup->vertex[1+i].z = POWERUP_RADIUS + POWERUP_RADIUS * RACK_Z;
	}

	for (i = 0; i < 5; i++) {
		powerup->vertex[6+i].x = powerup->x + 0.5f + POWERUP_RADIUS * RACK_R * sin(phi + 3.14159265358979323846f*2/5 * i);
		powerup->vertex[6+i].y = powerup->y + 0.5f + POWERUP_RADIUS * RACK_R * cos(phi + 3.14159265358979323846f*2/5 * i);
		powerup->vertex[6+i].z = POWERUP_RADIUS - POWERUP_RADIUS * RACK_Z;
	}

	powerup->vertex[11].x = powerup->x + 0.5f;
	powerup->vertex[11].y = powerup->y + 0.5f;
	powerup->vertex[11].z = 0;

	{
		float uv[12][2];
		uv[0][0] = uv[11][0] = 0.5f;
		uv[0][1] = uv[11][1] = 0.5f;
		for (i = 0; i < 5; i++) {
			uv[1+i][0] = 0.5f + 0.4999f * sin(2*phi + 3.14159265358979323846f/5 * (1+2*i));
			uv[1+i][1] = 0.5f + 0.4999f * cos(2*phi + 3.14159265358979323846f/5 * (1+2*i));
			uv[6+i][0] = 0.5f + 0.4999f * sin(2*phi + 3.14159265358979323846f*2/5 * i);
			uv[6+i][1] = 0.5f + 0.4999f * cos(2*phi + 3.14159265358979323846f*2/5 * i);
		}
		for (i = 0; i < 20; i++) {
			powerup->polygon[i].p[0].u = uv[pwrpvtx[i][0]][0];
			powerup->polygon[i].p[0].v = uv[pwrpvtx[i][0]][1];
			powerup->polygon[i].p[1].u = uv[pwrpvtx[i][1]][0];
			powerup->polygon[i].p[1].v = uv[pwrpvtx[i][1]][1];
			powerup->polygon[i].p[2].u = uv[pwrpvtx[i][2]][0];
			powerup->polygon[i].p[2].v = uv[pwrpvtx[i][2]][1];
		}
	}

	for (i = 0; i < 20; i++) {
		POLYGON *polygon = &powerup->polygon[i];
		prepare_polygon(polygon);
		add_temp_to_bsp_tree(tree, polygon);
	}
}
#undef RACK_R
#undef RACK_Z


void prepare_powerup_vertices(MATRIX_f *camera, POWERUP *powerup)
{
	int i;
	for (i = 0; i < 12; i++) draw_prepare_vertex(camera, &powerup->vertex[i]);
}
