#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "explode.h"


/*           0
 *
 *           3         <-- back
 *    4             2
 *      5         1     <-- front
 *
 *      9         8    <-- back
 *   10             7
 *           6          <-- front
 *
 *          11
 */


static const unsigned char expvtx[20][3] = {
	{0,1,2}, {0,2,3}, {0,3,4}, {0,4,5}, {0,5,1},
	{5,6,1}, {1,7,2}, {2,8,3}, {3,9,4}, {4,10,5},
	{1,6,7}, {2,7,8}, {3,8,9}, {4,9,10}, {5,10,6},
	{6,11,7}, {7,11,8}, {8,11,9}, {9,11,10}, {10,11,6}
};


static void init_explosion_polygons(EXPLOSION *explosion)
{
	int i;
	for (i = 0; i < 20; i++) {
		explosion->polygon[i].n_vtx = 3;
		explosion->polygon[i].polytype = POLYTYPE_GCOL;
		explosion->polygon[i].texture = NULL;
		explosion->polygon[i].p = &explosion->polyvtx[i*3];
		explosion->polyvtx[i*3  ].vtx = &explosion->vertex[expvtx[i][0]];
		explosion->polyvtx[i*3+1].vtx = &explosion->vertex[expvtx[i][1]];
		explosion->polyvtx[i*3+2].vtx = &explosion->vertex[expvtx[i][2]];
	}
}


void add_explosion_polygons(BSP_TREE *tree, EXPLOSION *explosion)
#define RACK_Z 0.4f
#define RACK_R 0.9f
{
	int i;

	explosion->vertex[0].x = explosion->x;
	explosion->vertex[0].y = explosion->y;
	explosion->vertex[0].z = explosion->z + explosion->rad[0];

	for (i = 0; i < 5; i++) {
		explosion->vertex[1+i].x = explosion->x + explosion->rad[1+i] * RACK_R * sin(3.14159265358979323846f/5 * (1+2*i));
		explosion->vertex[1+i].y = explosion->y + explosion->rad[1+i] * RACK_R * cos(3.14159265358979323846f/5 * (1+2*i));
		explosion->vertex[1+i].z = explosion->z + explosion->rad[1+i] * RACK_Z;
	}

	for (i = 0; i < 5; i++) {
		explosion->vertex[6+i].x = explosion->x + explosion->rad[6+i] * RACK_R * sin(3.14159265358979323846f*2/5 * i);
		explosion->vertex[6+i].y = explosion->y + explosion->rad[6+i] * RACK_R * cos(3.14159265358979323846f*2/5 * i);
		explosion->vertex[6+i].z = explosion->z - explosion->rad[6+i] * RACK_Z;
	}

	explosion->vertex[11].x = explosion->x;
	explosion->vertex[11].y = explosion->y;
	explosion->vertex[11].z = explosion->z - explosion->rad[11];

	{
		int vtxcol[12];
		for (i = 0; i < 12; i++) {
			float t = explosion->time[i] / explosion->maxmag;
			if (t > 0)
				vtxcol[i] = (int)floor(FLOAT_COLOUR_EXPLOSION_DARK +
					(FLOAT_COLOUR_EXPLOSION_BRIGHT - FLOAT_COLOUR_EXPLOSION_DARK) * sqrt(t));
			else
				vtxcol[i] = 0;
		}
		for (i = 0; i < 20; i++) {
			explosion->polygon[i].p[0].c = vtxcol[expvtx[i][0]];
			explosion->polygon[i].p[1].c = vtxcol[expvtx[i][1]];
			explosion->polygon[i].p[2].c = vtxcol[expvtx[i][2]];
		}
	}

	for (i = 0; i < 20; i++) {
		POLYGON *polygon = &explosion->polygon[i];
		if (polygon->p[0].c | polygon->p[1].c | polygon->p[2].c) {
			if (!polygon->p[0].c) polygon->p[0].c = (int)FLOAT_COLOUR_EXPLOSION_DARK;
			if (!polygon->p[1].c) polygon->p[1].c = (int)FLOAT_COLOUR_EXPLOSION_DARK;
			if (!polygon->p[2].c) polygon->p[2].c = (int)FLOAT_COLOUR_EXPLOSION_DARK;
			prepare_polygon(polygon);
			add_temp_to_bsp_tree(tree, polygon);
		}
	}
}
#undef RACK_R
#undef RACK_Z


void prepare_explosion_vertices(MATRIX_f *camera, EXPLOSION *explosion)
{
	int i;
	for (i = 0; i < 12; i++) draw_prepare_vertex(camera, &explosion->vertex[i]);
}


void create_explosion(GAME *game, float x, float y, float z, float mag)
{
	EXPLOSION *explosion = malloc(sizeof(*explosion));
	int i;
	if (!explosion) return;
	explosion->next = game->explosion;
	game->explosion = explosion;
	explosion->x = x;
	explosion->y = y;
	explosion->z = z;
	explosion->maxmag = mag;
	for (i = 0; i < 12; i++) {
		explosion->time[i] = explosion->mag[i] = mag * (1.0f - RND(0.5f));
		explosion->rad[i] = 0;
	}
	init_explosion_polygons(explosion);
}


void update_explosions(GAME *game)
{
	EXPLOSION **explosionp = &game->explosion;
	while (*explosionp) {
		EXPLOSION *explosion = *explosionp;
		int i;
		int spent = 1;
		for (i = 0; i < 12; i++) {
			float t;
			explosion->time[i] -= 0.01f;
			if (explosion->time[i] > 0) spent = 0;
			t = 1.0f - explosion->time[i] / explosion->mag[i];
			/* t is 1 for outside, 0 for centre */
			explosion->rad[i] = (float)pow(t, 0.25) * explosion->mag[i];
		}
		if (spent) {
			*explosionp = explosion->next;
			free(explosion);
		} else
			explosionp = &explosion->next;
	}
}
