#ifndef INC_GAME_H
#define INC_GAME_H


#include "bsp.h"


typedef struct POD POD;

#define POD_STATE_DEAD      0
#define POD_STATE_THRUSTING 1
#define POD_STATE_FALLING   2

#define POD_IS_THERE(pod) ((pod)->state != POD_STATE_DEAD)
#define POD_INVINC_TIME 3000
#define POD_CLOAK_TIME 3000

#define POD_FIRE_DELAY 20
#define POD_HEALTH_INITIAL 0x10000
#define POD_HEALTH_INITIAL_SHIFT 16

/* flags */
#define POD_EXTERNAL_VIEW 1

struct POD
{
	POD *next;
	fixed x, y, z, phi;
	fixed xv, yv, zv, phiv;
	int flags;
	unsigned char id;
	unsigned char state;
	fixed health;
	int invinc_time;
	int cloak_time;
	int fire_delay; /* doubles as respawn delay */
	const int *view_dim; /* pointer to array of four ints: left, top, width, height; 4x4 for whole screen */

	POLYGON polygon[7*8 + 1 + 7];
	POLYGON_VERTEX polyvtx[3*8 + 4*6*8 + 8 + 6*4 + 6];
	VERTEX vertex[1 + 7*8 + 12];
};


typedef struct BULLET BULLET;

struct BULLET
{
	BULLET *next;
	fixed x, y, z;
	fixed xv, yv, zv;

	POLYGON polygon[6];
	POLYGON_VERTEX polyvtx[6*4];
	VERTEX vertex[8];
};


typedef struct EXPLOSION EXPLOSION;

struct EXPLOSION
{
	EXPLOSION *next;
	float x, y, z;
	float maxmag;
	float mag[12];
	float rad[12];
	float time[12];
	POLYGON polygon[20];
	POLYGON_VERTEX polyvtx[3*20];
	VERTEX vertex[12];
};


typedef struct POWERUP POWERUP;

#define POWERUP_N_FRAMES 1024
#define POWERUP_RADIUS 0.5f
#define FIXED_POWERUP_RADIUS 0x8000

#define POWERUP_EXTRA_HEALTH  0
#define POWERUP_INVINCIBILITY 1
#define POWERUP_CLOAK         2
#define POWERUP_N             3

struct POWERUP
{
	POWERUP *next;
	int x, y;
	unsigned short frame;
	unsigned char type;
	POLYGON polygon[20];
	POLYGON_VERTEX polyvtx[3*20];
	VERTEX vertex[12];
};


#define TILE_CLEAR 0
#define TILE_WALL 128

#define TILE_IS_HIGH(t) (t & 128)

#define NEW_POWERUP_TIME 1500
#define MAX_POWERUPS 4

typedef struct GAME
{
	int map_w, map_h;
	unsigned char **map;
	POD *pod;
	BULLET *bullet;
	EXPLOSION *explosion;
	POWERUP *powerup;
	int new_powerup_time;
	int n_powerups;

	BSP_TREE *bsp_tree;
	int n_polygons;
	POLYGON *polygon;
	int n_vertices;
	VERTEX *vertex;
}
GAME;


#define PHYSICAL_WALL_HEIGHT 1.0f
#define FIXED_WALL_HEIGHT 0x10000
#define CAMERA_HEIGHT (0.85f * POD_RADIUS)
#define CAMERA_FORWARD (0.3f * POD_RADIUS)
#define EXT_CAMERA_HEIGHT 1.2f
#define EXT_CAMERA_FORWARD (-2.5f * POD_RADIUS)
#define FIXED_GRAVITY 0x20
#define FIXED_HOVER_MIN 0x8000
#define FIXED_HOVER_MAX 0x8100
#define FIXED_POD_RADIUS 0x7C00
#define FIXED_POD_HEIGHT FIXED_POD_RADIUS
#define FIXED_POD_FULL_HEIGHT ((int)(FIXED_POD_HEIGHT * 1.3f))
#define POD_RADIUS (FIXED_POD_RADIUS / 65536.0f)
#define POD_HEIGHT POD_RADIUS /* actually the height of the centre of the ball on top */


GAME *create_game(int n_players);
void destroy_game(GAME *game);

void draw_game(void *data);
int update_game(void *data);


#endif /* INC_GAME_H */
